#!/usr/bin/env python

from moinformat import make_input, make_linker, make_output, make_parser, \
                       make_serialiser, make_theme, parse, serialise
from os.path import split
import sys

def getmapping(mappings):
    mapping = {}
    key = None

    for arg in mappings:
        if key is None:
            key = arg
        else:
            mapping[key] = arg
            key = None

    return mapping

def getvalue(values):
    return values and values[0] or None

def main():
    dirname, progname = split(sys.argv[0])
    args = sys.argv[1:]

    # Option values.

    l = filenames = []
    formats = []
    input_dir_types = []
    input_dirs = []
    input_encodings = []
    input_page_seps = []
    mappings = []
    output_dirs = []
    output_encodings = []
    theme_names = []
    pagenames = []

    # Flags.

    all = False
    fragment = False
    macros = False
    tree = False

    for arg in args:

        # Detect tree output.

        if arg == "--tree":
            tree = True

        # Detect macro evaluation.

        elif arg == "--macros":
            macros = True

        # Detect all documents.

        elif arg == "--all":
            all = True

        # Detect fragment output (if serialising).

        elif arg == "--fragment":
            fragment = True

        # Switch to collecting formats.

        elif arg == "--format":
            l = formats
            continue

        # Switch to collecting input locations.

        elif arg == "--input-dir":
            l = input_dirs
            continue

        # Switch to collecting input context types.

        elif arg == "--input-dir-type":
            l = input_dir_types
            continue

        # Switch to collecting input encodings.

        elif arg == "--input-encoding":
            l = input_encodings
            continue

        # Switch to collecting input page hierarchy separators.

        elif arg == "--input-page-sep":
            l = input_page_seps
            continue

        # Switch to collecting mappings.

        elif arg == "--mapping":
            l = mappings
            continue

        # Switch to collecting output locations.

        elif arg == "--output-dir":
            l = output_dirs
            continue

        # Switch to collecting output encodings.

        elif arg == "--output-encoding":
            l = output_encodings
            continue

        # Switch to collecting page names.

        elif arg == "--pagename":
            l = pagenames
            continue

        # Switch to collecting theme names.

        elif arg == "--theme":
            l = theme_names
            continue

        # Collect options and arguments.

        else:
            l.append(arg)

            # Collect multiple mappings.

            if l is mappings:
                continue

        # Collect filenames normally.

        l = filenames

    format = formats and formats[0] or "html"

    # Derive a proper mapping from the given list of values.

    mapping = getmapping(mappings)

    # Obtain encodings.

    input_encoding = getvalue(input_encodings)
    output_encoding = getvalue(output_encodings)

    # Obtain the input and output locations and contexts.

    input_dir = getvalue(input_dirs)
    output_dir = getvalue(output_dirs)

    input_page_sep = getvalue(input_page_seps)

    input_context = input_dir and (getvalue(input_dir_types) or
                                   "directory") or "standalone"

    input = make_input(input_context, {"encoding" : input_encoding,
                                       "filename" : input_dir,
                                       "separator" : input_page_sep})

    output_context = output_dir and "directory" or "standalone"

    output = make_output(output_context, {"encoding" : output_encoding,
                                          "filename" : output_dir})

    # Obtain a theme name.

    theme_name = not fragment and (getvalue(theme_names) or "default") or None

    theme = None

    # Treat filenames as pagenames if an input directory is indicated and if no
    # pagenames are explicitly specified.

    if input_dir:
        if pagenames:
            print >>sys.stderr, """\
Explicit pagenames (indicated using --pagename) are only to be specified when
providing filenames without an input directory (indicated using --input-dir).

To indicate pagenames within an input directory, omit any --pagename flags."""
            sys.exit(1)

        if all:
            if filenames:
                print >>sys.stderr, """\
Using --all overrides any indicated pagenames. Either --all or the filenames
should be omitted."""
                sys.exit(1)
            else:
                filenames = input.all()

        pagenames = filenames
        filenames = []

    # Open each file or page, parse the content, serialise the document.

    for pagename, filename in map(None, pagenames, filenames):

        # Define a pagename if missing.

        pagename = pagename or split(filename)[-1]

        # Read either from a filename or using a pagename.

        if filename:
            pagetext = input.readfile(filename)
        else:
            pagetext = input.readpage(pagename)

        # Parse the page content.

        p = make_parser()
        d = parse(pagetext, p)

        if macros:
            p.evaluate_macros()

        # Show a document tree for debugging purposes, if requested.

        if tree:
            print d.prettyprint()
            continue

        # Otherwise, serialise the document.

        # Obtain a linker using format and pagename details.

        linker = make_linker(format, pagename, mapping)

        # Obtain a serialiser using the configuration.

        serialiser = make_serialiser(format, output, linker, pagename)
        outtext = serialise(d, serialiser)

        # Obtain a theme object for theming.

        theme = theme_name and make_theme("%s.%s" % (theme_name, format),
                                          output, linker, pagename)

        # With a theme, apply it to the text.

        if theme:
            outtext = theme.apply(outtext)

        # If reading from a file, show the result. Otherwise, write to the
        # output context.

        if not output.can_write():
            print outtext
        else:
            output.writepage(outtext, pagename)
            print >>sys.stderr, pagename

    # Install any theme resources.

    if theme:
        theme.install_resources()

if __name__ == "__main__":
    main()

# vim: tabstop=4 expandtab shiftwidth=4
